/* 
 *  Copyright (C) 2004 Girish Ramakrishnan All Rights Reserved.
 *
 * This is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this software; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
 * USA.
 */

// $Id: customtraylabel.h,v 1.10 2007/01/02 21:48:37 cs19713 Exp $

#ifndef _CUSTOMTRAYLABEL_H
#define _CUSTOMTRAYLABEL_H

#include <tqobject.h>
#include <tqaction.h>
#include "qtraylabel.h"

class TQStringList;
class TQPopupMenu;
class TDEPopupMenu;
class TQString;
class TQSettings;
class TQWidget;
class TQDropEvent;

class CustomTrayLabel : public TQTrayLabel
{
  Q_OBJECT

public:
  CustomTrayLabel(Window w, TQWidget* p = 0, const TQString& t = TQString::null);
  CustomTrayLabel(const TQStringList& argv, pid_t pid, TQWidget* parent = 0);

  // Session management
  bool saveState(TDEConfig *config);
  bool restoreState(TDEConfig *config);

  bool isDockWhenObscured(void) const { return mDockWhenObscured->isOn(); }
  
  void setAppName(const TQString& name);

public slots:
  // overridden to update our menu
  void setDockWhenMinimized(bool dwm);
  void setDockWhenFocusLost(bool dfl) { mDockWhenFocusLost->setOn(dfl); }
  void setSkipTaskbar(bool skip);

  void setDockWhenObscured(bool dock) { mDockWhenObscured->setOn(dock); }
  void enableSessionManagement(bool sm);
  
protected:
  void dropEvent(TQDropEvent *ev);
  bool canUnsubscribeFromRoot(void);
  void mapEvent(void);
  void focusLostEvent();
  void obscureEvent(void);
  void destroyEvent(void);
  void mouseReleaseEvent(TQMouseEvent * ev);
  bool canDockWindow(Window w);
  void processDead(void);

private slots:
  void setCustomIcon(void);
  void updateMenu();
  void slotSetBalloonTimeout(void);
  
private:
  void installMenu();
  bool mUndockWhenDead;
  TQPopupMenu *mMainMenu;
  TDEPopupMenu *mOptionsMenu;
  TQAction *mDockOnRestore, *mBalloonTimeout, *mSkipTaskbar, *mDockWhenMinimized,
           *mDockWhenObscured, *mSessionManagement, *mDockWhenFocusLost;
  int mShowId;
};

#endif // _CUSTOMTRAYLABEL_H
